"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessoryInfoServiceInitializer = exports.SmartSpeakerServiceInitializer = exports.BaseServiceInitializer = void 0;
class BaseServiceInitializer {
    constructor(logger, hap, alexaBridge) {
        this.logger = logger;
        this.hap = hap;
        this.alexaBridge = alexaBridge;
    }
    initialize(accessory, device, media) {
        const serviceType = this.getServiceType();
        let service = accessory.getService(serviceType);
        if (!service) {
            service = accessory.addService(serviceType);
        }
        this.init(service, device, media);
    }
}
exports.BaseServiceInitializer = BaseServiceInitializer;
class SmartSpeakerServiceInitializer extends BaseServiceInitializer {
    getServiceType() {
        return this.hap.Service.SmartSpeaker;
    }
    init(service, device, media) {
        this.setCurrentMediaState(service, media.currentState);
        this.setTargetMediaState(service, media.currentState);
        this.setVolume(service, media.volume);
        this.setMute(service, media.muted);
        const HapCharacteristic = this.hap.Characteristic;
        const TargetMediaState = HapCharacteristic.TargetMediaState;
        const Volume = HapCharacteristic.Volume;
        const Mute = HapCharacteristic.Mute;
        this.alexaBridge.onCharacteristicSet(device, service, TargetMediaState, value => {
            switch (value) {
                case TargetMediaState.PAUSE:
                case TargetMediaState.STOP:
                    return 'pause';
                case TargetMediaState.PLAY:
                    return 'play';
            }
            throw new Error(`Unexpected value: ${JSON.stringify(value)}`);
        });
        this.alexaBridge.onCharacteristicSet(device, service, Volume, 'volume', value => value);
        this.alexaBridge.onCharacteristicSet(device, service, Mute, 'mute', value => {
            return { mute: value };
        });
        this.alexaBridge.onDeviceCommand(device, command => {
            if (this.isVolumeChange(command)) {
                if (null !== command.payload.volumeSetting) {
                    this.setVolume(service, command.payload.volumeSetting);
                }
                if (null !== command.payload.isMuted) {
                    this.setMute(service, command.payload.isMuted);
                }
            }
            else if (this.isAudioPlayerState(command)) {
                this.setCurrentMediaState(service, command.payload.audioPlayerState);
                this.setTargetMediaState(service, command.payload.audioPlayerState);
            }
        });
    }
    setVolume(service, volume) {
        service.getCharacteristic(this.hap.Characteristic.Volume).setValue(volume);
    }
    setMute(service, isMuted) {
        service.getCharacteristic(this.hap.Characteristic.Mute).setValue(isMuted);
    }
    setCurrentMediaState(service, state) {
        const CurrentMediaState = this.hap.Characteristic.CurrentMediaState;
        let currentMediaState;
        switch (state) {
            case 'PLAYING':
                currentMediaState = CurrentMediaState.PLAY;
                break;
            case 'INTERRUPTED':
                currentMediaState = CurrentMediaState.PAUSE;
                break;
            case 'FINISHED':
            default:
                currentMediaState = CurrentMediaState.STOP;
                break;
        }
        service.getCharacteristic(CurrentMediaState).setValue(currentMediaState);
    }
    setTargetMediaState(service, state) {
        const TargetMediaState = this.hap.Characteristic.TargetMediaState;
        let targetMediaState;
        switch (state) {
            case 'PLAYING':
                targetMediaState = TargetMediaState.PLAY;
                break;
            case 'INTERRUPTED':
                targetMediaState = TargetMediaState.PAUSE;
                break;
            case 'FINISHED':
            default:
                targetMediaState = TargetMediaState.STOP;
                break;
        }
        service.getCharacteristic(TargetMediaState).setValue(targetMediaState);
    }
    isVolumeChange(command) {
        return command.command === "PUSH_VOLUME_COMMAND" /* PUSH_VOLUME_COMMAND */;
    }
    isAudioPlayerState(command) {
        return command.command === "PUSH_AUDIO_PLAYER_STATE" /* PUSH_AUDIO_PLAYER_STATE */;
    }
}
exports.SmartSpeakerServiceInitializer = SmartSpeakerServiceInitializer;
class AccessoryInfoServiceInitializer extends BaseServiceInitializer {
    getServiceType() {
        return this.hap.Service.AccessoryInformation;
    }
    init(service, device, media) {
        // TODO: const identify = service.getCharacteristic(this.Characteristic.Identify);
        const manufacturer = service.getCharacteristic(this.hap.Characteristic.Manufacturer);
        const model = service.getCharacteristic(this.hap.Characteristic.Model);
        const name = service.getCharacteristic(this.hap.Characteristic.Name);
        const serialNumber = service.getCharacteristic(this.hap.Characteristic.SerialNumber);
        const firmwareRevision = service.getCharacteristic(this.hap.Characteristic.FirmwareRevision);
        manufacturer.setValue('Amazon.com');
        model.setValue(device.deviceType);
        firmwareRevision.setValue(device.softwareVersion);
        name.setValue(device.accountName);
        serialNumber.setValue(device.serialNumber);
        // TODO: Handle device name changes and software updates.
    }
}
exports.AccessoryInfoServiceInitializer = AccessoryInfoServiceInitializer;
